<?php

/**
 * Menu callback; Retrieve a JSON object containing autocomplete suggestions
 * for existing ACLs defined by UAC module.
 */
function uac_ui_acl_autocomplete($string = '') {
  $matches = array();
  if ($string) {
    $result = db_select('acl')->fields('acl', array('name'))->condition('name', db_like($string) . '%', 'LIKE')->condition('module', 'uac', '=')->range(0, 10)->execute();
    foreach ($result as $acl) {
      $matches[$acl->name] = check_plain($acl->name);
    }
  }

  drupal_json_output($matches);
}

/**
 * Menu callback; Administration page for groups
 */
function uac_ui_admin_acls() {
  $acls = uac_get_acls();

  $form['acls'] = array(
    '#tree' => TRUE,
  );

  foreach ($acls as $acl_id => $name) {
    $form['acls'][$acl_id]['#acl'] = (object) array(
      'acl_id' => $acl_id,
      'name' => $name,
    );
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#title_display' => 'invisible',
    '#size' => 32,
    '#maxlength' => 64,
  );
  $form['add'] = array(
    '#type' => 'submit',
    '#value' => t('Add UAC group'),
    '#validate' => array('uac_ui_admin_acl_validate'),
    '#submit' => array('uac_ui_admin_acl_submit'),
  );

  return $form;
}

/**
 * Form to configure a single ACL.
 *
 * @ingroup forms
 * @see uac_ui_admin_acl_validate()
 * @see uac_ui_admin_acl_submit()
 */
function uac_ui_admin_acl($form, $form_state, $acl) {
  // Display the edit acl form.
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Group name'),
    '#default_value' => $acl->name,
    '#size' => 30,
    '#required' => TRUE,
    '#maxlength' => 64,
    '#description' => t('The name for this group. Example: "news editor", "premium visitor".'),
  );
  $form['acl_id'] = array(
    '#type' => 'value',
    '#value' => $acl->acl_id,
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save UAC group'),
  );
  $form['actions']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete UAC group'),
    '#submit' => array('uac_ui_admin_acl_delete_submit'),
  );

  return $form;
}

/**
 * Form validation handler for uac_ui_admin_acl().
 */
function uac_ui_admin_acl_validate($form, &$form_state) {
  if (!empty($form_state['values']['name'])) {
    if ($form_state['values']['op'] == t('Save UAC group')) {
      $acl_id = uac_get_acl_id_by_name($form_state['values']['name']);
      if ($acl_id && $acl_id != $form_state['values']['acl_id']) {
        form_set_error('name', t('The group name %name already exists. Choose another group name.', array('%name' => $form_state['values']['name'])));
      }
    }
    elseif ($form_state['values']['op'] == t('Add UAC group')) {
      if (uac_get_acl_id_by_name($form_state['values']['name'])) {
        form_set_error('name', t('The group name %name already exists. Choose another group name.', array('%name' => $form_state['values']['name'])));
      }
    }
  }
  else {
    form_set_error('name', t('You must specify a valid group name.'));
  }
}

/**
 * Form submit handler for uac_ui_admin_acl().
 */
function uac_ui_admin_acl_submit(&$form, &$form_state) {
  $acl = (object) $form_state['values'];
  if ($form_state['values']['op'] == t('Save UAC group')) {
    uac_save_acl($acl);
    drupal_set_message(t('The UAC group has been renamed.'));
  }
  elseif ($form_state['values']['op'] == t('Add UAC group')) {
    uac_save_acl($acl);
    drupal_set_message(t('The UAC group has been added.'));
  }
  uac_save_acl($acl);

  $form_state['redirect'] = 'admin/people/uac';
  return;
}

/**
 * Form submit handler for the uac_ui_admin_acl() form.
 */
function uac_ui_admin_acl_delete_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/people/uac/delete/' . $form_state['values']['acl_id'];
}

/**
 * Form to confirm role delete operation.
 */
function uac_ui_admin_acl_delete_confirm($form, &$form_state, $acl) {
  $form['acl_id'] = array(
    '#type' => 'value',
    '#value' => $acl->acl_id,
  );
  return confirm_form($form, t('Are you sure you want to delete the UAC group %name ?', array('%name' => $acl->name)), 'admin/people/uac', t('This action cannot be undone.'), t('Delete'));
}

/**
 * Form submit handler for uac_ui_admin_acl_delete_confirm().
 */
function uac_ui_admin_acl_delete_confirm_submit($form, &$form_state) {
  acl_delete_acl((int) $form_state['values']['acl_id']);
  drupal_set_message(t('The UAC group has been deleted.'));
  $form_state['redirect'] = 'admin/people/uac';
}

/**
 * Returns HTML for the groups form.
 *
 * @param $variables
 *   An associative array containing:
 *   - form: A render element representing the form.
 *
 * @ingroup themeable
 */
function theme_uac_ui_admin_acls($variables) {
  $form = $variables['form'];

  $header = array(t('Name'), array('data' => t('Operations'), 'colspan' => 2));
  foreach (element_children($form['acls']) as $acl_id) {
    $name = $form['acls'][$acl_id]['#acl']->name;
    $row = array();
    $row[] = check_plain($name);
    $row[] = l(t('edit group'), 'admin/people/uac/edit/' . $acl_id);
    $row[] = l(t('delete group'), 'admin/people/uac/delete/' . $acl_id);
    $rows[] = array('data' => $row);
  }
  $rows[] = array(array('data' => drupal_render($form['name']) . drupal_render($form['add']), 'colspan' => 3, 'class' => 'edit-name'));

  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'uac-ui-acls')));
  $output .= drupal_render_children($form);

  return $output;
}

